/***************************************************************************

    M.A.M.E.CE  -  Multiple Arcade Machine Emulator for WinCE
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse
	WinCE Portions Copyright (C) 1999,2000 Benjamin Cooley, Glenn Neidermeier
    
    This file is part of MAMECE, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

 ***************************************************************************/

/***************************************************************************

  CESound.c

 ***************************************************************************/

#include "mame.h"
//#include "osdepend.h"
//#include "mame32.h"
//#include "CESound.h"

// GN: redid the includes due to compile problems on workstation
#include <windows.h>
#include "options.h"
#include "osd_cpu.h"
#include "sound.h"
#include "driver.h"


/***************************************************************************
    function prototypes
 ***************************************************************************/

static int      CESound_init(options_type *options);
static void     CESound_exit(void);
static void     CESound_update_audio(void);
static void     CESound_play_sample(int channel, signed char* data, int len, int freq, int volume, int loop);
static void     CESound_play_sample_16(int channel, signed short* data, int len, int freq, int volume, int loop);
static void     CESound_play_streamed_sample(int channel, signed char* data, int len, int freq, int volume, int pan);
static void     CESound_play_streamed_sample_16(int channel, signed short* data, int len, int freq, int volume, int pan);
static void     CESound_set_sample_freq(int channel,int freq);
static void     CESound_set_sample_volume(int channel,int volume);
static void     CESound_stop_sample(int channel);
static void     CESound_restart_sample(int channel);
static int      CESound_get_sample_status(int channel);
static void     CESound_set_mastervolume(int volume);
static int      CESound_get_mastervolume(void);
static void     CESound_sound_enable(int enable);

// GN: new V36 functions
int     CESound_start_audio_stream(int stereo);
int     CESound_update_audio_stream(INT16* buffer);
void    CESound_stop_audio_stream(void);

/***************************************************************************
    External variables
 ***************************************************************************/

struct OSDSound CESound = 
{
    { CESound_init },                     //    int     (*init)(options_type *options);
    { CESound_exit },                     //    void    (*exit)(void);
    { CESound_start_audio_stream },       //    int     (*start_audio_stream)(int stereo);
    { CESound_update_audio_stream },      //    int     (*update_audio_stream)(INT16* buffer);
    { CESound_stop_audio_stream },        //    void    (*stop_audio_stream)(void);
    { CESound_set_mastervolume },         //    void    (*set_mastervolume)(int attenuation);
    { CESound_get_mastervolume },         //    int     (*get_mastervolume)(void);
    { CESound_sound_enable },             //    void    (*sound_enable)(int enable);
    { CESound_update_audio }              //    void    (*update_audio)(void);
};

/***************************************************************************
    Internal structures
 ***************************************************************************/

#define NUM_WAVEHDRS  4

struct tSound_private
{
	WAVEOUTCAPS		m_Caps;
	HWAVEOUT		m_hWaveOut;
	WAVEHDR			m_WaveHdrs[NUM_WAVEHDRS];

	int				m_nSampleRate;
	int				m_nSampleBits;
	int				m_nChannels;
	int				m_nVolume;		// -32 to 0 attenuation value

    int             m_nSamplesPerFrame; // GN
    int             m_nBytesPerFrame;   // GN
};

/***************************************************************************
    Internal variables
 ***************************************************************************/

static struct tSound_private      This;

/***************************************************************************
    External OSD functions  
 ***************************************************************************/

static  int CESound_init(options_type *options)
{
	// GN: I hardwired in samplerate=11025 in EnablePlayOptions()

	MMRESULT res;
	WAVEFORMATEX wf;


	This.m_hWaveOut = NULL;

	if (Machine->sample_rate == 0)
	{
		Machine->sample_rate = options->sample_rate;
	}

	This.m_nSampleRate = Machine->sample_rate;
	This.m_nSampleBits = 16; // Machine->sample_bits;
	This.m_nChannels = 1;

	wf.wFormatTag = WAVE_FORMAT_PCM;
	wf.nChannels = This.m_nChannels; 
	wf.nSamplesPerSec = This.m_nSampleRate; 
	wf.nAvgBytesPerSec = This.m_nSampleRate * This.m_nSampleBits / 8; 
	wf.nBlockAlign = This.m_nSampleBits * This.m_nChannels / 8;
	wf.wBitsPerSample = This.m_nSampleBits; 
	wf.cbSize = 0;

	res = waveOutOpen(
		&This.m_hWaveOut,	// Handle
		WAVE_MAPPER, 		// ID (0 for wave mapper)
		&wf,				// Wave format
		0,					// Callback
		0,					// Instance data
		CALLBACK_NULL);

	if (res != MMSYSERR_NOERROR)
		return 1;

	memset(&This.m_WaveHdrs, 0, sizeof(WAVEHDR) * NUM_WAVEHDRS);

	This.m_nVolume = 0;


	return 0;
}


////////////////////////////////////////////////
static void CESound_exit(void)
{
	BOOL done;
	int i, ticks;

	ticks = GetTickCount();
	done = FALSE;
	while (!done && GetTickCount() - ticks < 500)
	{
		done = TRUE;
		for (i = 0; i < NUM_WAVEHDRS; i++)
		{
			if (This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				This.m_WaveHdrs[i].lpData = NULL;
				This.m_WaveHdrs[i].dwBufferLength = 0;
				This.m_WaveHdrs[i].dwFlags = 0;
			}
			else if (This.m_WaveHdrs[i].dwFlags)
				done = FALSE;
		}
	}

	waveOutClose(This.m_hWaveOut);
}


//
DWORD g_add, g_end;
////////////////////////////////////////////////////
int     CESound_start_audio_stream(int stereo)
{
	int i; // TEST

    if (Machine->sample_rate == 0)
        return 0;

    if (stereo)
        stereo = 1;	/* make sure it's either 0 or 1 */

    /* determine the number of samples and bytes per frame */
    This.m_nSamplesPerFrame = (double)Machine->sample_rate / Machine->drv->frames_per_second;
    This.m_nBytesPerFrame   = This.m_nSamplesPerFrame * sizeof(INT16) * (stereo + 1);



/// TEST

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
	    int newlen = This.m_nBytesPerFrame; // len
		{
			This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen << 1);
			This.m_WaveHdrs[i].dwBufferLength = newlen;
			This.m_WaveHdrs[i].dwFlags = 0;
		}
	}
	g_add = (((DWORD)(This.m_nSampleRate << 15) / (DWORD)This.m_nSampleRate) << 1) + 3;
	g_end = This.m_nBytesPerFrame << 16;
/// end test


	return This.m_nSamplesPerFrame;
}


////////////////////////////////////////////////////////
int     CESound_update_audio_stream(INT16* buffer)
{
    signed short *data = buffer;
    int newlen = This.m_nBytesPerFrame; // len
	int freq = This.m_nSampleRate;

	int i;
	short *s;
    short *d;

	DWORD pos, add, end;
	int rate;

	rate = This.m_nSampleRate; // * 100 / 75; // GN: not sure why this was done

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
//			int newlen = len * rate / freq;
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != newlen)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
#if TEST
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
#endif
			}

#if TEST
			if (!This.m_WaveHdrs[i].lpData)
#endif
			{
#if TEST
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen << 1);
				This.m_WaveHdrs[i].dwBufferLength = newlen;
				This.m_WaveHdrs[i].dwFlags = 0;
#endif
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}

			s = data;
			d = (short *)This.m_WaveHdrs[i].lpData;

#if TEST
			add = (((DWORD)(freq << 15) / (DWORD)rate) << 1) + 3;
			end = newlen << 16;
#endif

//			for (pos = 0; pos < end; pos += add)
			for (pos = 0; pos < g_end; pos += g_add)
			{
//				DWORD abspos = pos >> 16;
//				short samp1 = s[abspos];
//				*d++ = samp1;
				*d++ = s[pos >> 16];
			}

			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return This.m_nSamplesPerFrame;
		}
	}
	return 0; // GN: doesn't really return here usually
}


//////////////////////////////////////////////////
void    CESound_stop_audio_stream(void)
{
}

/////////////////////////////////////////////////
static void CESound_set_mastervolume(int volume)
{
	This.m_nVolume = volume;
}

/////////////////////////////////////////////////
static int CESound_get_mastervolume(void)
{
    return This.m_nVolume;
}

/////////////////////////////////////////////////
static void CESound_sound_enable(int enable)
{
}

/////////////////////////////////////////////////
static void CESound_update_audio(void)
{
}





#if USED // legacy routines

void CESound_play_sample(int channel, signed char *data, int len, int freq, int volume, int loop)
{
	int i;
	signed char *s; // GN // signed short *s;
	signed short *d;
	DWORD pos, add, end;

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
			int newlen = len * This.m_nSampleRate / freq;
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != newlen)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
			}
			if (!This.m_WaveHdrs[i].lpData)
			{
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen << 1);
				This.m_WaveHdrs[i].dwBufferLength = newlen;
				This.m_WaveHdrs[i].dwFlags = 0;
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}
			s = data;
			d = (signed short *)This.m_WaveHdrs[i].lpData;
			add = (((DWORD)(freq << 15) / (DWORD)This.m_nSampleRate) << 1) + 3;
			end = len << 16;
			for (pos = 0; pos < end; pos += add)
				*d++ = (unsigned char)((int)s[(pos >> 16)] + 128);
			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return;	
		}
	}

}

static void CESound_play_sample_16(int channel, signed short *data, int len, int freq, int volume, int loop)
{
	int i;
	signed short *s;
	signed short *d;
	DWORD pos, add, end;

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
			int newlen = len * This.m_nSampleRate / freq;
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != newlen)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
			}
			if (!This.m_WaveHdrs[i].lpData)
			{
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen << 1);
				This.m_WaveHdrs[i].dwBufferLength = newlen;
				This.m_WaveHdrs[i].dwFlags = 0;
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}
			s = data;
			d = (signed short *)This.m_WaveHdrs[i].lpData;
			add = (((DWORD)(freq << 15) / (DWORD)This.m_nSampleRate) << 1) + 3;
			end = len << 16;
			for (pos = 0; pos < end; pos += add)
				*d++ = (signed char)s[(pos >> 16)];
			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return;	
		}
	}
}

static void CESound_play_streamed_sample(int channel, signed char *data, int len, int freq, int volume, int pan)
{
	int i;
	signed char *s;
	unsigned char *d;
	DWORD pos, add, end;
	int rate;

	rate = This.m_nSampleRate * 100 / 75;

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
			int newlen = len * rate / freq;
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != newlen)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
			}
			if (!This.m_WaveHdrs[i].lpData)
			{
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen);
				This.m_WaveHdrs[i].dwBufferLength = newlen;
				This.m_WaveHdrs[i].dwFlags = 0;
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}
			s = data;
			d = (unsigned char *)This.m_WaveHdrs[i].lpData;
			add = (((DWORD)(freq << 15) / (DWORD)rate) << 1) + 3;
			end = len << 16;
			for (pos = 0; pos < end; pos += add)
				*d++ = (unsigned char)((int)s[(pos >> 16)] + 128);
			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return;	
		}
	}
}

static void CESound_play_streamed_sample_16(int channel, signed short *data, int len, int freq, int volume, int pan)
{
	int i;
	signed short *s;
	signed short *d;
	DWORD pos, add, end;
	int rate;

	rate = This.m_nSampleRate * 100 / 75;

	for (i = 0; i < NUM_WAVEHDRS; i++)
	{
		if (This.m_WaveHdrs[i].dwFlags == 0 || This.m_WaveHdrs[i].dwFlags & WHDR_DONE)
		{
			int newlen = len * rate / freq;
			if (This.m_WaveHdrs[i].lpData && This.m_WaveHdrs[i].dwBufferLength != newlen)
			{
				waveOutUnprepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
				free(This.m_WaveHdrs[i].lpData);
				This.m_WaveHdrs[i].lpData = NULL;
			}
			if (!This.m_WaveHdrs[i].lpData)
			{
				This.m_WaveHdrs[i].lpData = (LPSTR)malloc(newlen << 1);
				This.m_WaveHdrs[i].dwBufferLength = newlen;
				This.m_WaveHdrs[i].dwFlags = 0;
				waveOutPrepareHeader(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			}
			s = data;
			d = (signed short *)This.m_WaveHdrs[i].lpData;
			add = (((DWORD)(freq << 15) / (DWORD)rate) << 1) + 3;
			end = len << 16;
			for (pos = 0; pos < end; pos += add)
				*d++ = (signed char)s[(pos >> 16)];
			waveOutWrite(This.m_hWaveOut, &This.m_WaveHdrs[i], sizeof(WAVEHDR));
			return;	
		}
	}
}

static void CESound_set_sample_freq(int channel,int freq)
{
}

static void CESound_set_sample_volume(int channel,int volume)
{
}

static void CESound_stop_sample(int channel)
{
}

static void CESound_restart_sample(int channel)
{
}

static int CESound_get_sample_status(int channel)
{
    return 0;
}

#endif // GN
