/***************************************************************************

    M.A.M.E.32  -  Multiple Arcade Machine Emulator for Win32
    Win32 Portions Copyright (C) 1997-98 Michael Soderstrom and Chris Kirmse
    
    This file is part of MAME32, and may only be used, modified and
    distributed under the terms of the MAME license, in "readme.txt".
    By continuing to use, modify or distribute this file you indicate
    that you have read the license and understand and accept it fully.

GAN - 26APR00 - deleted most of the file so I could use FindRomSet()

***************************************************************************/
 
/***************************************************************************

  Properties.c

    Properties Popup and Misc UI support routines.
    
    Created 8/29/98 by Mike Haaland (mhaaland@hypertech.com)

***************************************************************************/

#define MULTI_MONITOR

#include "driver.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>
#include <commdlg.h>
#include <strings.h>
#include <sys/stat.h>
//GN #include <ddraw.h>

#include "info.h"
//GN #include "audit32.h"
#include "options.h"
#include "file.h"
#include "resource.h"
#include "Joystick.h"       /* For joystick avalibility. */
#include "DIJoystick.h"     /* For DIJoystick avalibility. */
#include "DIKeyboard.h"     /* For DIKeyboard avalibility. */
#include "DDrawDisplay.h"   /* For display modes. */
#include "m32util.h"
//GN #include "directdraw.h"
#include "properties.h"

#include "Mame32.h"
#include "DataMap.h"
#include "resource.hm"

//GN #include <multimon.h>



/***************************************************************
 * Public functions
 ***************************************************************/


/* Checks of all ROMs are available for 'game' and returns result
 * Returns TRUE if all ROMs found, 0 if any ROMs are missing.
 */
BOOL FindRomSet (int game)
{
    const struct RomModule  *romp;
    const struct GameDriver *gamedrv;
    const char              *name;
    int                     count = 0;
	int						err;
	unsigned int			length, icrc;

    gamedrv = drivers[game];
    romp = gamedrv->rom;

    if (!osd_faccess (gamedrv->name, OSD_FILETYPE_ROM))
    {
        /* if the game is a clone, try loading the ROM from the main version */
        if (gamedrv->clone_of == 0
        ||  gamedrv->clone_of == DRIVER_ROOT
        ||  !osd_faccess(gamedrv->clone_of->name, OSD_FILETYPE_ROM))
            return FALSE;
    }

    while (romp->name || romp->offset || romp->length)
    {
        if (romp->name || romp->length)
            return FALSE; /* expecting ROM_REGION */

        romp++;

        while (romp->length)
        {
            const struct GameDriver *drv;

            if (romp->name == 0)
                return FALSE;   /* ROM_CONTINUE not preceeded by ROM_LOAD */
            else
            if (romp->name == (char *)-1)
                return FALSE;   /* ROM_RELOAD not preceeded by ROM_LOAD */

            name = romp->name;
            count++;
			icrc = romp->crc;
			length = 0;
			count++;

			/* obtain CRC-32 and length of ROM file */
			drv = gamedrv;
			do
			{
				err = osd_fchecksum(drv->name, name, &length, &icrc);
				drv = drv->clone_of;
			} while (err && drv && drv != DRIVER_ROOT);

			if (err)
				return FALSE;

            do
            {
                romp++;
            }
            while (romp->length && (romp->name == 0 || romp->name == (char *)-1));
        }
    }
    return TRUE;
}

/* Checks if the game uses external samples at all
 * Returns TRUE if this driver expects samples
 */
BOOL GameUsesSamples(int game)
{
    int i;
    static const struct GameDriver *gamedrv;

    gamedrv = drivers[game];
    
	for( i = 0; gamedrv->drv->sound[i].sound_type && i < MAX_SOUND; i++ )
    {
        const char **samplenames = NULL;

        if( gamedrv->drv->sound[i].sound_type != SOUND_SAMPLES )
            continue;

        samplenames = ((struct Samplesinterface *)gamedrv->drv->sound[i].sound_interface)->samplenames;
		if (samplenames != 0 && samplenames[0] != 0)
            return TRUE;
    }
    return FALSE;
}

/* Checks for all samples in a sample set.
 * Returns TRUE if all samples are found, FALSE if any are missing.
 */
BOOL FindSampleSet (int game)
{
    static const struct GameDriver *gamedrv;
    const char  *sharedname;
    BOOL bStatus;
    int  skipfirst;
    int  count = 0;
    int  j, i;
    
    if (GameUsesSamples(game) == FALSE)
        return TRUE;

    gamedrv = drivers[game];
    
	for( i = 0; gamedrv->drv->sound[i].sound_type && i < MAX_SOUND; i++ )
    {
        const char **samplenames = NULL;

        if( gamedrv->drv->sound[i].sound_type != SOUND_SAMPLES )
			continue;
        
        samplenames = ((struct Samplesinterface *)gamedrv->drv->sound[i].sound_interface)->samplenames;

		if (samplenames != 0 && samplenames[0] != 0)
        {
            BOOL have_samples = FALSE;
            BOOL have_shared  = FALSE;

            if (samplenames[0][0]=='*')
            {
                sharedname = samplenames[0]+1;
                skipfirst = 1;
            }
            else
            {
                sharedname = NULL;
                skipfirst = 0;
            }

            /* do we have samples for this game? */
            have_samples = osd_faccess (gamedrv->name, OSD_FILETYPE_SAMPLE);

            /* try shared samples */
            if (skipfirst)
                have_shared = osd_faccess (sharedname, OSD_FILETYPE_SAMPLE);

            /* if still not found, we're done */
            if (!have_samples && !have_shared)
                return FALSE;
				
            for (j = skipfirst; samplenames[j] != 0; j++)
            {
                bStatus = FALSE;

                /* skip empty definitions */
                if (strlen (samplenames[j]) == 0)
                    continue;

                if (have_samples)
                    bStatus = File_Status (gamedrv->name, samplenames[j], OSD_FILETYPE_SAMPLE);

                if (!bStatus && have_shared)
                {
                    bStatus = File_Status (sharedname, samplenames[j], OSD_FILETYPE_SAMPLE);
                    if (!bStatus)
                    {
                        return FALSE;
                    }
                }
            }
        }
    }
    return TRUE;
}

