// UI module for MamePPC, 
// Various portions appropriated from MameCE and Mame32 projects
// by G. Neidermeier 4/2000

// defines
#define WIN32_LEAN_AND_MEAN

// includes
#include <windows.h>
#include "resource.h"

#include "driver.h"
#include "file.h"
#include "mame32.h"
#include "m32util.h"
#include "cgamex.h"
#include "Properties.h"

// forwards
static void MamePlayGameWithOptions(void);
static BOOL Win32UI_init(HINSTANCE hInstance, LPTSTR lpCmdLine);
static long WINAPI MameWindowProc(HWND hWnd,UINT message,UINT wParam,LONG lParam);
static BOOL PumpMessage();
static BOOL PumpAndReturnMessage(MSG *pmsg);
static BOOL MameCommand(HWND hwnd,int id, HWND hwndCtl, UINT codeNotify);
static void EnablePlayOptions(int nIndex, options_type *o);
static long /* INT_PTR */ CALLBACK LoadErrorDialogProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam);
static char *GetRomList(int iGame);
static BOOL GameCheck(void);

// globals
static options_type playing_game_options;
static HWND hwndList;
static HWND hwndRotate;
static HWND hMain;
static HACCEL hAccel;
int  game_count;
int  play_game;
BOOL gSoundEn = FALSE; // my global for sound enable
int gRotate = ROTATE_NONE; // my global for rotation option
static int  game_index;

////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE    hInstance,
                   HINSTANCE    hPrevInstance,
                   LPTSTR        lpCmdLine,
                   int          nCmdShow)
{
    MSG     msg;
	play_game = -1; // 0; // -1;
	game_index = 0;

    if (!Win32UI_init(hInstance, lpCmdLine))
        return 1;

#ifndef NO_GAMEX
	if (!GAMEX_Init())
	{
		ErrorMsg("Your CE device doesn't support direct screen access.  "
				 "MAMECE requires a DSA compatible device to run.");
		return 1;
	}
#endif

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
			TranslateMessage(&msg);
			DispatchMessage(&msg);

			if (play_game >= 0)
			{
				MamePlayGameWithOptions();
				play_game = -1;
			}
	}

#ifndef NO_GAMEX
	GAMEX_Close();
#endif

	return msg.wParam;
}




//////////////////////////////////////////////////////////////

static BOOL PumpMessage()
{
    MSG msg;

    return PumpAndReturnMessage(&msg);
}


//////////////////////////////////////////////////////////////

static BOOL PumpAndReturnMessage(MSG *pmsg)
{
    if (!(GetMessage(pmsg, NULL, 0, 0)))
        return FALSE;
 
    if (!TranslateAccelerator(hMain, hAccel, pmsg))
    {
        if (!IsDialogMessage(hMain,pmsg))
        {
            TranslateMessage(pmsg);
            DispatchMessage(pmsg);
        }
    }
    return TRUE;
}




////////////////////////////////////////////////////////////////

static BOOL Win32UI_init(HINSTANCE hInstance, LPTSTR lpCmdLine)
{
	TCHAR buf[100];


	/* Initialize game list */
    game_count = 0;
    while (drivers[game_count] != 0)
        game_count++;

	OptionsInit(game_count);
    File.init();

	// Create the main dlg
	hMain = CreateDialog (hInstance, 
				MAKEINTRESOURCE(IDD_MAME), 0, MameWindowProc) ;
    hwndList        = GetDlgItem(hMain, IDC_LIST);


	// Put the items in the Rotate Combo Box
    //#define ROTATE_NONE  0
    //#define ROTATE_RIGHT 1
    //#define ROTATE_LEFT  2
	hwndRotate		= GetDlgItem(hMain, IDC_ROTATE);
    SendMessage(hwndRotate, CB_RESETCONTENT, 0, 0); 
    SendMessage(hwndRotate, CB_ADDSTRING, 0, (LPARAM)TEXT("None") ); 
    SendMessage(hwndRotate, CB_ADDSTRING, 0, (LPARAM)TEXT("Right") ); 
    SendMessage(hwndRotate, CB_ADDSTRING, 0, (LPARAM)TEXT("Left") ); 

	/* Initialize game list */
    game_count = 0;
    while (drivers[game_count] != 0)
	{
		BOOL GotGame = FALSE;
		game_index = game_count; // game_index used by GameCheck
#ifndef DEBUG
		GotGame = GameCheck();
#endif // this is slow, needs to happen in it's own thread
		if (GotGame)
			wsprintf(buf, TEXT("+ %S"), drivers[game_count]->description );
		else
			wsprintf(buf, TEXT("- %S"), drivers[game_count]->description );

		SendMessage(hwndList, LB_ADDSTRING, 0, (WPARAM)buf);
        game_count++;
	}

    // AdjustMetrics();
	ShowWindow(hMain, SW_SHOW);

	return TRUE;
}


////////////////////////////////////////////////////////////////////////*/

static long WINAPI MameWindowProc(HWND hWnd,UINT message,UINT wParam,LONG lParam)
{
	int wmEvent = HIWORD(wParam); 
	
	switch(message)
	{
	case WM_CLOSE:
		DestroyWindow(hWnd);
		return TRUE;
		break;
		
	case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDC_SOUNDEN:
				if (wmEvent == BN_CLICKED)
					gSoundEn = BST_CHECKED & 
							(SendDlgItemMessage(hWnd, IDC_SOUNDEN, BM_GETSTATE,0, 0) ); 
				break;
			case IDC_LIST:
				if (wmEvent == LBN_DBLCLK)
					play_game = SendMessage(hwndList, LB_GETCURSEL, 0, 0);
				break;
			default:
				break;
			}	// end switch(LOWORD(wParam))
			
			break; // end case WM_COMMAND
		}
		
	case WM_DESTROY:
		PostQuitMessage(0);
		break;	// end case WM_DESTROY
		
	}	// end switch (message)
	
	return FALSE; 
	//	DefWindowProc(hWnd, message, wParam, lParam);
}



////////////////////////////////////////

static void MamePlayGameWithOptions()
{
	HWND hPrevWnd = NULL;	// GN: can we find the UI window after playing?

    int index;

    index = play_game; // GetSelectedPickItem();

    memcpy(&playing_game_options, GetGameOptions(index), sizeof(options_type));
    
    /* Deal with options that can be disabled. */   
    EnablePlayOptions(index, &playing_game_options);

    /* based on our raw options, convert the ones that we can into
       regular MAME config options */

    if (playing_game_options.error_log)
        options.errorlog = fopen("error.log", "wa");
    else
        options.errorlog = NULL;

#ifdef MAME_DEBUG
    options.mame_debug = 1;
#else
    options.mame_debug = 0;
#endif
    options.cheat       = 0; // playing_game_options.cheat;
    options.gui_host    = 1;

    /*
        Need sample rate initialised _before_ rom loading
        for optional rom regions.
    */
    if (playing_game_options.sound == SOUND_NONE)
    {
        /* update the Machine structure to show that sound is disabled */
		Machine->sample_rate = 0;
		options.samplerate = 0;
    }
    else
    {
        options.samplerate  = playing_game_options.sample_rate;
    }

// GN: V36 //    options.samplebits          = playing_game_options.sample_bits;
// GN: V36 //    options.no_fm               = 1;
    options.use_samples         = playing_game_options.use_samples;
    options.use_emulated_ym3812 = (playing_game_options.fm_ym3812 == 0);
    options.norotate            = 0;  

    switch (playing_game_options.rotate)
    {
    case ROTATE_RIGHT:
        options.ror = 1;
        options.rol = 0;
        break;

    case ROTATE_LEFT:
        options.ror = 0;
        options.rol = 1;
        break;

    default:
        options.ror = 0;
        options.rol = 0;
        break;
    }
    options.flipx = 0; // playing_game_options.flipx;
    options.flipy = 0; // playing_game_options.flipy;

    options.beam   = (int)(1 /*playing_game_options.beam*/ * 0x00010000);
    if (options.beam < 0x00010000)    /* beam MUST be at least 1 pixel */
        options.beam = 0x00010000;
    if (options.beam > 0x00100000)    /* set max to 16 pixels */
        options.beam = 0x00100000;

    options.flicker   = (int)(100 /*playing_game_options.flicker*/ * 2.55);
    if (options.flicker < 0)
        options.flicker = 0;
    if (options.flicker > 255)
        options.flicker = 255;

    options.translucency   = playing_game_options.translucency;
    options.antialias      = playing_game_options.antialias;;
    options.use_artwork    = playing_game_options.use_artwork;

//    if (joygui != NULL)
//        KillTimer(hMain,0);

//    sprintf(buf, "%s - MAME32", drivers[index]->description);
//    SetWindowText(hMain, buf);

//    abort_loading = FALSE;
//    hLoad = CreateDialog(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_LOAD_PROGRESS),hMain,
//                         LoadProgressDialogProc);

//    sprintf(buf, "Loading %s", drivers[index]->description);
//    SetWindowText(hLoad, buf);

//    GetWindowRect(hMain, &rect);

//    in_emulation = TRUE;

    if (run_game(index) == 0)
    {
//       IncrementPlayCount(index);
//       ListView_RedrawItems(hwndList,GetSelectedPickItem(),GetSelectedPickItem());
    }
    else
    {
//        if (abort_loading == FALSE)
        { 
//            ShowWindow(hMain, SW_SHOW);
//            EndDialog(hLoad,0);
            DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_ROM_ERROR), NULL, LoadErrorDialogProc);
//MessageBox(NULL, TEXT("Rom Error"), TEXT("MAME 32"), MB_OK);
        }
    }

//    in_emulation = FALSE;
    
//    if (quit)
//    {
//       PostMessage(hMain, WM_CLOSE, 0, 0);
//       return;
//    }


    // resort if sorting on # of times played
//    if (abs(GetSortColumn()) - 1 == COLUMN_PLAYED)
//	{
//	}



//    SetWindowText(hMain, "MAME32");
//    UpdateStatusBar();

    ShowWindow(hMain, SW_SHOW);


	// GN: can we find UI window after playing?
	hPrevWnd = FindWindow(NULL, TEXT("MAME32"));
	if (hPrevWnd) 
	{
		SetForegroundWindow(hPrevWnd);
	}



//    SetFocus(hwndList);
    
//    SetWindowPos(hMain, 0, 0, 0, rect.right - rect.left,
//        rect.bottom - rect.top,
//        SWP_NOMOVE | SWP_NOZORDER | SWP_NOCOPYBITS | SWP_SHOWWINDOW);

//    if (joygui != NULL)
//        SetTimer(hMain,0,25,NULL);

    if (playing_game_options.error_log && options.errorlog)
        fclose(options.errorlog);
}


//////////////////////////////////////////

static /* INT_PTR*/ long CALLBACK LoadErrorDialogProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	TCHAR errmsg[50];
	char *romlst;
	char *token;
	HWND hwndErrList;
	
    switch (Msg)
    {
    case WM_INITDIALOG:
		//        iGame = GetSelectedPickItem();
		//        CenterSubDialog(hMain,hDlg,FALSE);
		//        Static_SetText(GetDlgItem(hDlg, IDC_PROP_TITLE), GameInfoTitle(iGame));
		//        Edit_SetText(GetDlgItem(hDlg, IDC_ERROR_DETAILS), GetRomList(iGame));

		// GN: setup error msg
		SetForegroundWindow(hDlg);

#ifdef _WIN32_WCE
        wsprintf(errmsg, TEXT("Error loading %S"), drivers[play_game]->description);
#else
        sprintf(errmsg, "Error loading %s", drivers[play_game]->description);
#endif
		SetDlgItemText(hDlg, IDC_PROP_TITLE, errmsg );
		
		/* Setup Rom list display */
		hwndErrList = GetDlgItem(hDlg, IDC_ERROR_DETAILS);
		romlst = GetRomList(play_game);

		token = strtok(romlst, "\n");

		while( token != NULL )
		{
#ifdef _WIN32_WCE
			wsprintf(errmsg, TEXT("%S"), token );
#else
			sprintf(errmsg, "%s", token );
#endif
			SendMessage(hwndErrList, LB_ADDSTRING, 
				0, (WPARAM)errmsg );
			
			token = strtok( NULL, "\n" );
		}
        return 1;
		
    case WM_COMMAND :
		if (LOWORD(wParam) == IDOK)
		{
            EndDialog(hDlg,1);
		}
		return 1;
	case WM_CLOSE:
        EndDialog(hDlg,1);
		return 1;
	}
    return 0;
}



//////////////////////////////////////////

static char *GetRomList(int iGame)
{
    static char buf[5000];
    const struct RomModule *romp =  drivers[iGame]->rom;

    buf[0] = '\0';

    if (NULL == romp)
        return buf;

    while (romp->name || romp->offset || romp->length)
    {
        romp++; /* skip memory region definition */
        
        while (romp->length)
        {
            char name[100];
            int length;
            
            sprintf(name, romp->name, drivers[iGame]->name);
            
            length = 0;
            
            do
            {
                /* ROM_RELOAD */
                if (romp->name == (char *)-1)
                    length = 0; /* restart */
                
                length += romp->length & ~ROMFLAG_MASK;
                
                romp++;
            } while (romp->length && (romp->name == 0 || romp->name == (char *)-1));
            
//            sprintf(buf + strlen(buf), "%s:\t%-12s\t%u bytes\r\n", drivers[iGame]->name, name, length);
            sprintf(buf + strlen(buf), "%s\n", name);
        }
    }
    return buf;
}


//////////////////////////////////////////

static BOOL GameCheck(void)
{

//    LV_FINDINFO lvfi;
//    int         i;
    BOOL        success;
    BOOL        changed = FALSE;

//    if (game_index == 0)
//        ProgressBarShow();

//    if (game_count <= game_index)
//    {
//        bDoGameCheck = FALSE;
//        ProgressBarHide();
//        return FALSE;
//    }
     
    if (GetHasRoms(game_index) == UNKNOWN)
    {
        if (NULL == drivers[game_index]->rom)
        {
            success = TRUE;
        }
        else 
        {
            /* Only check for the ZIP file once */
            success = File_ExistZip(drivers[game_index]->name,OSD_FILETYPE_ROM);
        }

        if (!success)
            success = FindRomSet(game_index);

        SetHasRoms(game_index, success);
        changed = TRUE;
    }

    /* Check for SAMPLES */
//    if (GetHasSamples(game_index) == UNKNOWN)
    {
//        SetHasSamples(game_index, FindSampleSet(game_index));
//        changed = TRUE;
    }

//    lvfi.flags = LVFI_PARAM;
//    lvfi.lParam = game_index;

//    if (changed && (i = ListView_FindItem(hwndList, -1, &lvfi)) != -1);
//        ListView_RedrawItems(hwndList, i, i);
//    if ((game_index % progBarStep) == 0)
//        ProgressBarStep();
//    game_index++;

	return success; // GN
    return changed;
}




//////////////////////////////////////////

/* Adjust options - tune them to the currently selected game */
static void EnablePlayOptions(int nIndex, options_type *o)
{
	// GN: change options here from defaults as appropriate
	o->rotate = 0;
	o->di_keyboard = FALSE;
	o->is_window = TRUE;
	o->window_ddraw = FALSE;
	o->di_joystick = FALSE;
	o->use_ai_mouse = FALSE;
	o->scale = 1; // This.m_bDouble in GDIdisplay
//#ifndef _WIN32_WCE
	o->sound = SOUND_MIDAS; // GN: took over the Midas slot for CE audio subsystem
//#endif

	if (!gSoundEn)
		o->sound = SOUND_NONE;

	o->rotate = SendMessage(hwndRotate, CB_GETCURSEL, 0, 0); 
	o->sample_rate = 11025;
}


//////////////////////////////////////////

options_type * GetPlayingGameOptions()
{
    return &playing_game_options;
}



///////////////////////////////////////////

int UpdateLoadProgress(const char *name, int current,int total)
{
	// make main ui window go away
    ShowWindow(hMain,SW_HIDE);
    return 0;
}



///////////////////////////////////////////

void _assert(int exp)
{
#ifndef WIN32
	MessageBox(0, _T("Assertion failure detected"), _T("ASSERT"), MB_ICONSTOP | MB_OK);
#endif
}

